from colorama import Fore, Back, Style
import shutil
import datetime


def line(
    info: str = None,
    width: int = 80,
    char: str = "─",
    color: str = "magenta",
    align: int = 1,
    margin: tuple = (1, 1),
    *args,
):

    columns, _ = shutil.get_terminal_size()
    sup_limit = columns
    if width == 0 or width > columns:
        width = sup_limit
    if align < 0:
        info = "" if info is None else info
        align = (
            width + align - len(info) - sum(margin)
        ) % width
    else:
        align = align % width

    color_code = getattr(Fore, color.upper())
    if info is None:
        return str(
            color_code
            + str(str(char) * width)[0:width]
            + Fore.RESET
        )
    sub_limit = len(info) + sum(margin) + align
    if width < sub_limit:
        width = sub_limit
    else:
        blank_chars_length = (
            width - sum(margin) - len(info) - align
        )
        return "".join(
            [
                color_code,
                str(char * align)[0:align],
                " " * margin[0],
                info,
                " " * margin[1],
                str(char * blank_chars_length)[
                    0:blank_chars_length
                ],
                Fore.RESET,
            ]
        )


def dateline(
    width: int = 80,
    char: str = "─",
    color: str = "magenta",
    align: int = 1,
    margin: tuple = (1, 1),
    *args,
):
    return line(
        datetime.datetime.now().strftime(
            "%d %b %Y %H:%M:%S"
        ),
        width,
        char,
        color,
        align,
        margin,
        *args,
    )



import unittest
from seperator.lines import line


class TestLinesFunctions(unittest.TestCase):
    def test_empty_call(self):
        self.assertEqual(
            line(), "\x1b[35m" + 80 * "─" + "\x1b[39m"
        )

    def test_info(self):
        self.assertEqual(
            line("INFORMATIVE TEXT"),
            "\x1b[35m"
            + "─"
            + " "
            + "INFORMATIVE TEXT"
            + " "
            + "─"
            * (80 - (2 + 1 + len("INFORMATIVE TEXT")))
            + "\x1b[39m",
        )

    def test_color(self):
        self.assertEqual(
            line(color="ReD"),
            "\x1b[31m" + 80 * "─" + "\x1b[39m",
        )

    def test_align(self):
        self.assertEqual(
            line("INFORMATIVE TEXT", align=-1),
            "\x1b[35m"
            + "─"
            * (80 - (2 + 1 + len("INFORMATIVE TEXT")))
            + " "
            + "INFORMATIVE TEXT"
            + " "
            + "─"
            + "\x1b[39m",
        )

    def test_margin(self):
        self.assertEqual(
            line("INFORMATIVE TEXT", margin=(2, 3)),
            "\x1b[35m"
            + "─"
            + " " * 2
            + "INFORMATIVE TEXT"
            + " " * 3
            + "─"
            * (80 - (5 + 1 + len("INFORMATIVE TEXT")))
            + "\x1b[39m",
        )

    def test_char(self):
        self.assertEqual(
            line(char="#"),
            "\x1b[35m" + 80 * "#" + "\x1b[39m",
        )

    def test_triple_char(self):
        self.assertEqual(
            line("a", char="####"),
            "\x1b[35m"
            + "# "
            + "a"
            + " "
            + 76 * "#"
            + "\x1b[39m",
        )
        self.assertEqual(len(line(char="#")), 90)
        self.assertEqual(len(line(char="-#")), 90)
        self.assertEqual(len(line(char="-*#")), 90)
        self.assertEqual(len(line(char="**#")), 90)
        self.assertEqual(len(line(char="*#")), 90)
        self.assertEqual(
            len(line("sadfasdf ", char="*#")), 90
        )
        self.assertEqual(
            len(line(char="asdfasdfasf#")), 90
        )


if __name__ == "__main__":
    unittest.main()
